@@ -1,9 +1,14 @@
 # -*- coding: utf-8 -*-
 from __future__ import (unicode_literals, absolute_import)
-from money import Money
+
+import logging
 
 import requests
 import xmltodict
+from money import Money
+
+
+logger = logging.getLogger(__name__)
 
 
 class Error(object):
@@ -23,8 +28,17 @@ class Costs(dict):
                                 currency='EUR')
         self.shipping = Money(amount=float(raw_dict['Porto']) / 100,
                               currency='EUR')
-        self.tax_free = Money(amount=float(raw_dict['Porto_MwSt_Frei']) / 100,
-                              currency='EUR')
+        self.shipping_tax_free = Money(amount=float(raw_dict['Porto_MwSt_Frei']) / 100,
+                                       currency='EUR')
+
+
+class OrderException(Exception):
+    def __init__(self, errors):
+        super(OrderException, self).__init__(
+            ' - '.join(e.message for e in errors)
+        )
+
+        self.errors = errors
 
 
 class Order(object):
@@ -48,6 +62,9 @@ class Order(object):
 
     URL = 'https://www.briefdruckzentrum.de/rest/bdz/auftrag'
 
+    # ignore error: 900 TESTAUFTRAG
+    IGNORE_ERROR_CODES = (900, )
+
     errors = None
 
     @classmethod
@@ -87,6 +104,7 @@ class Order(object):
                 self.errors = [Error(e) for e in r_dict['Error']['Error']]
             else:
                 self.errors = [Error(r_dict['Error']['Error'])]
+
         if 'Files' in r_dict:
             self.files = r_dict['Files']
         if "Auftragskosten" in r_dict:
@@ -94,23 +112,36 @@ class Order(object):
 
 
 class Client(object):
-    CREATE_ORDER_URL = ''
-    GET_ACCOUNT_BALANCE_URL = 'https://www.briefdruckzentrum.de/rest/bdz/kontostand'
-    CREATE_ACCOUNT_DEPOSIT_URL = 'https://www.briefdruckzentrum.de/rest/bdz/kontoeinzahlung'
-
     def __init__(self, user, password):
-        self.auth = user, password
+        self.session = requests.Session()
+        self.session.auth = (user, password)
+        self.session.verify = True
 
     def create_order(self, file, color_mode, region,
                      duplex=Order.DUPLEX, paper=0, envelopeDL=0, envelopeC4=0,
                      envelope_format=Order.ENVELOPE_DL, name=None, test=True):
+
         url, data, files = Order.get_request(file, color_mode, region, duplex,
                                              paper, envelopeDL, envelopeC4,
                                              envelope_format, name, test)
 
-        response = requests.post(url, data=data, files=files, auth=self.auth)
-        if response.status_code != 200:
-            response.raise_for_status()
+        logger.debug("url %s, data %s", url, data)
+
+        response = self.session.post(url, data=data, files=files)
+        logger.debug("response %s %s", response.status_code, response.text)
+
+        response.raise_for_status()
+
         order = Order(response.text)
+        if order.errors:
+            real_errors = [
+                error
+                for error in order.errors
+                if error.code not in Order.IGNORE_ERROR_CODES
+            ]
+
+            if real_errors:
+                raise OrderException(real_errors)
+
         order.request = response.request
         return order
